@@ -35,7 +35,7 @@
 
 #include "fs-private.h"
 
-#define GST_CAT_DEFAULT fs_conference_debug
+#define GST_CAT_DEFAULT _fs_conference_debug
 
 /**
  * SECTION:fs-codec
