@@ -149,7 +149,7 @@
 #include "fs-enumtypes.h"
 #include "fs-private.h"
 
-#define GST_CAT_DEFAULT fs_conference_debug
+#define GST_CAT_DEFAULT _fs_conference_debug
 
 /* Signals */
 enum
