@@ -23,6 +23,7 @@
 # include <config.h>
 #endif
 
+#include <gst/audio/audio.h>
 #include <gst/check/gstcheck.h>
 #include <farstream/fs-conference.h>
 #include <farstream/fs-stream-transmitter.h>
@@ -129,7 +130,7 @@ setup_simple_conference_full (
 
   bus = gst_pipeline_get_bus (GST_PIPELINE (dat->pipeline));
   fail_if (bus == NULL);
-  gst_bus_set_sync_handler (bus, default_sync_handler, dat);
+  gst_bus_set_sync_handler (bus, default_sync_handler, dat, NULL);
   gst_object_unref (bus);
 
   dat->conference = gst_element_factory_make (conference_elem, NULL);
@@ -272,12 +273,10 @@ setup_fakesrc (struct SimpleTestConference *dat)
 }
 
 static gboolean
-pad_count_fold (gpointer pad, GValue *val, gpointer user_data)
+pad_count_fold (const GValue *item, GValue *val, gpointer user_data)
 {
   g_value_set_uint (val, g_value_get_uint (val) + 1);
 
-  gst_object_unref (pad);
-
   return TRUE;
 }
 
@@ -329,7 +328,8 @@ GST_START_TEST (test_rawconference_new)
   FsStreamDirection dir;
 
   dat = setup_simple_conference (1, "fsrawconference", "bob@127.0.0.1");
-  st = simple_conference_add_stream (dat, dat, "shm", 0, NULL);
+  //st = simple_conference_add_stream (dat, dat, "shm", 0, NULL);
+  st = simple_conference_add_stream (dat, dat, "rawudp", 0, NULL);
 
   g_object_get (dat->session,
       "id", &id,
@@ -484,12 +484,11 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
         {
           const GValue *value;
           FsError errorno;
-          const gchar *error, *debug;
+          const gchar *error;
 
           ts_fail_unless (
-              gst_implements_interface_check (GST_MESSAGE_SRC (message),
-                  FS_TYPE_CONFERENCE),
-              "Received farstream-error from non-farstream element");
+            FS_IS_CONFERENCE (GST_MESSAGE_SRC (message)),
+            "Received farstream-error from non-farstream element");
 
           ts_fail_unless (
               gst_structure_has_field_typed (s, "src-object", G_TYPE_OBJECT),
@@ -499,17 +498,13 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
               "farstream-error structure has no src-object field");
           ts_fail_unless (
               gst_structure_has_field_typed (s, "error-msg", G_TYPE_STRING),
-              "farstream-error structure has no src-object field");
-          ts_fail_unless (
-              gst_structure_has_field_typed (s, "debug-msg", G_TYPE_STRING),
-              "farstream-error structure has no src-object field");
+              "farstream-error structure has no error-msg field");
 
           value = gst_structure_get_value (s, "error-no");
           errorno = g_value_get_enum (value);
           error = gst_structure_get_string (s, "error-msg");
-          debug = gst_structure_get_string (s, "debug-msg");
 
-          ts_fail ("Error on BUS (%d) %s .. %s", errorno, error, debug);
+          ts_fail ("Error on BUS (%d) %s", errorno, error);
         }
         else if (gst_structure_has_name (s, "farstream-new-local-candidate"))
         {
@@ -518,9 +513,8 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
           const GValue *value;
 
           ts_fail_unless (
-              gst_implements_interface_check (GST_MESSAGE_SRC (message),
-                  FS_TYPE_CONFERENCE),
-              "Received farstream-error from non-farstream element");
+            FS_IS_CONFERENCE (GST_MESSAGE_SRC (message)),
+            "Received farstream-error from non-farstream element");
 
           ts_fail_unless (
               gst_structure_has_field_typed (s, "stream", FS_TYPE_STREAM),
@@ -548,9 +542,8 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
           const GValue *value;
 
           ts_fail_unless (
-              gst_implements_interface_check (GST_MESSAGE_SRC (message),
-                  FS_TYPE_CONFERENCE),
-              "Received farstream-error from non-farstream element");
+            FS_IS_CONFERENCE (GST_MESSAGE_SRC (message)),
+            "Received farstream-error from non-farstream element");
 
           ts_fail_unless (
               gst_structure_has_field_typed (s, "stream", FS_TYPE_STREAM),
@@ -587,10 +580,8 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
           const GValue *value;
 
           ts_fail_unless (
-              gst_implements_interface_check (GST_MESSAGE_SRC (message),
-                  FS_TYPE_CONFERENCE),
-              "Received farstream-current-send-codec-change from non-farstream"
-              " element");
+            FS_IS_CONFERENCE (GST_MESSAGE_SRC (message)),
+            "Received farstream-error from non-farstream element");
 
           ts_fail_unless (
               gst_structure_has_field_typed (s, "session", FS_TYPE_SESSION),
@@ -619,10 +610,8 @@ _bus_callback (GstBus *bus, GstMessage *message, gpointer user_data)
           const GValue *value;
 
           ts_fail_unless (
-              gst_implements_interface_check (GST_MESSAGE_SRC (message),
-                  FS_TYPE_CONFERENCE),
-              "Received farstream-local-candidates-prepared from non-farstream"
-              " element");
+            FS_IS_CONFERENCE (GST_MESSAGE_SRC (message)),
+            "Received farstream-error from non-farstream element");
 
           ts_fail_unless (
               gst_structure_has_field_typed (s, "stream", FS_TYPE_STREAM),
@@ -1005,10 +994,9 @@ set_initial_codecs (
 
   ts_fail_unless (codecs == NULL, "Shouldn't generate codecs codecs");
 
-  codec = fs_codec_new (0, "audio/x-raw-int,"
-      "endianness=(int)1234, signed=(bool)true, "
-      "width=(int)16, depth=(int)16, "
-      "rate=(int)44100", FS_MEDIA_TYPE_AUDIO, 0);
+  codec = fs_codec_new (0, "audio/x-raw,"
+      "format=(string)"GST_AUDIO_NE (S16)","
+      "rate=(int)44100, channels=(int)1", FS_MEDIA_TYPE_AUDIO, 0);
   codecs = g_list_append (codecs, codec);
 
   filtered_codecs = g_list_append (filtered_codecs, codecs->data);
@@ -1388,112 +1376,6 @@ GST_START_TEST (test_rawconference_dispose)
 }
 GST_END_TEST;
 
-static void unref_session_on_src_pad_added (FsStream *stream,
-    GstPad *pad, FsCodec *codec, struct SimpleTestStream *st)
-{
-  TEST_LOCK ();
-
-  gst_element_set_locked_state (st->dat->fakesrc, TRUE);
-  gst_element_set_state (st->dat->fakesrc, GST_STATE_NULL);
-  gst_bin_remove (GST_BIN (st->dat->pipeline), st->dat->fakesrc);
-
-  ASSERT_CRITICAL (fs_session_destroy (st->dat->session));
-  st->dat->destroyed = TRUE;
-
-  TEST_UNLOCK ();
-
-  g_main_loop_quit (loop);
-}
-
-static void unref_session_init (struct SimpleTestStream *st, guint confid,
-    guint streamid)
-{
-  g_signal_connect (st->stream, "src-pad-added",
-      G_CALLBACK (unref_session_on_src_pad_added), st);
-}
-
-GST_START_TEST (test_rawconference_unref_session_in_pad_added)
-{
-  nway_test (2, NULL, unref_session_init, "rawudp", 0, NULL);
-}
-GST_END_TEST;
-
-static const gchar *signal_name;
-
-static GstBusSyncReply
-unref_stream_sync_handler (GstBus *bus, GstMessage *message,
-    gpointer data)
-{
-  struct SimpleTestConference *dat = data;
-  const GstStructure *s;
-  FsStream *stream;
-  const GValue *v;
-  GList *item;
-
-  if (GST_MESSAGE_TYPE (message) != GST_MESSAGE_ELEMENT)
-    return GST_BUS_PASS;
-
-  s = gst_message_get_structure (message);
-
-  if (!gst_structure_has_name (s, signal_name))
-    return GST_BUS_PASS;
-
-  v = gst_structure_get_value (s, "stream");
-  ts_fail_unless (G_VALUE_HOLDS (v, FS_TYPE_STREAM));
-  stream = g_value_get_object (v);
-
-  TEST_LOCK ();
-
-  for (item = dat->streams; item; item = item->next)
-  {
-    struct SimpleTestStream *st = item->data;
-    if (st->stream == stream)
-    {
-      fs_stream_destroy (stream);
-      st->destroyed = TRUE;
-      gst_message_unref (message);
-      g_main_loop_quit (loop);
-      TEST_UNLOCK ();
-      return GST_BUS_DROP;
-    }
-  }
-
-  TEST_UNLOCK ();
-
-  gst_message_unref (message);
-  return GST_BUS_DROP;
-}
-
-static void unref_stream_init (struct SimpleTestConference *dat, guint confid)
-{
-  GstBus *bus = gst_pipeline_get_bus (GST_PIPELINE (dat->pipeline));
-
-  gst_bus_set_sync_handler (bus, NULL, NULL);
-  gst_bus_set_sync_handler (bus, unref_stream_sync_handler, dat);
-  gst_object_unref (bus);
-}
-
-GST_START_TEST (test_rawconference_unref_stream_in_nice_thread_prepared)
-{
-  signal_name = "farstream-local-candidates-prepared";
-  nway_test (2, unref_stream_init, NULL, "nice", 0, NULL);
-}
-GST_END_TEST;
-
-GST_START_TEST (test_rawconference_unref_stream_in_nice_thread_new_active)
-{
-  signal_name = "farstream-new-active-candidate-pair";
-  nway_test (2, unref_stream_init, NULL, "nice", 0, NULL);
-}
-GST_END_TEST;
-
-GST_START_TEST (test_rawconference_unref_stream_in_nice_thread_state_changed)
-{
-  signal_name = "farstream-component-state-changed";
-  nway_test (2, unref_stream_init, NULL, "nice", 0, NULL);
-}
-GST_END_TEST;
-
 
 static Suite *
 fsrawconference_suite (void)
@@ -1511,7 +1393,7 @@ fsrawconference_suite (void)
 
   tc_chain = tcase_create ("fsrawconference_two_way_shm");
   tcase_add_test (tc_chain, test_rawconference_two_way_shm);
-  suite_add_tcase (s, tc_chain);
+  //suite_add_tcase (s, tc_chain);
 
   tc_chain = tcase_create ("fsrawconference_errors");
   tcase_add_test (tc_chain, test_rawconference_errors);
@@ -1533,27 +1415,6 @@ fsrawconference_suite (void)
   tcase_add_test (tc_chain, test_rawconference_dispose);
   suite_add_tcase (s, tc_chain);
 
-  tc_chain = tcase_create ("fsrawconference_unref_session_in_pad_added");
-  tcase_add_test (tc_chain, test_rawconference_unref_session_in_pad_added);
-  suite_add_tcase (s, tc_chain);
-
-  tc_chain = tcase_create (
-      "fsrawconference_unref_stream_in_nice_thread_prepared");
-  tcase_add_test (tc_chain,
-      test_rawconference_unref_stream_in_nice_thread_prepared);
-  suite_add_tcase (s, tc_chain);
-
-  tc_chain = tcase_create (
-      "fsrawconference_unref_stream_in_nice_thread_new_active");
-  tcase_add_test (tc_chain,
-      test_rawconference_unref_stream_in_nice_thread_new_active);
-  suite_add_tcase (s, tc_chain);
-
-  tc_chain = tcase_create (
-      "fsrawconference_unref_stream_in_nice_thread_state_changed");
-  tcase_add_test (tc_chain,
-      test_rawconference_unref_stream_in_nice_thread_state_changed);
-  suite_add_tcase (s, tc_chain);
   return s;
 }
 
