@@ -43,6 +43,8 @@
 #include <string.h>
 #include <sys/types.h>
 
+#include <gio/gio.h>
+
 #ifdef HAVE_UNISTD_H
 # include <unistd.h>
 #endif
@@ -89,7 +91,7 @@ struct _FsMulticastTransmitterPrivate
   GstElement **udpsrc_funnels;
   GstElement **udpsink_tees;
 
-  GMutex *mutex;
+  GMutex mutex;
   GList **udpsocks;
 
   gint type_of_service;
@@ -102,6 +104,11 @@ struct _FsMulticastTransmitterPrivate
   (G_TYPE_INSTANCE_GET_PRIVATE ((o), FS_TYPE_MULTICAST_TRANSMITTER, \
     FsMulticastTransmitterPrivate))
 
+#define FS_MULTICAST_TRANSMITTER_LOCK(self) \
+  g_mutex_lock (&(self)->priv->mutex);
+#define FS_MULTICAST_TRANSMITTER_UNLOCK(self) \
+  g_mutex_unlock (&(self)->priv->mutex);
+
 static void fs_multicast_transmitter_class_init (
     FsMulticastTransmitterClass *klass);
 static void fs_multicast_transmitter_init (FsMulticastTransmitter *self);
@@ -217,7 +224,7 @@ fs_multicast_transmitter_init (FsMulticastTransmitter *self)
   self->priv->disposed = FALSE;
 
   self->components = 2;
-  self->priv->mutex = g_mutex_new ();
+  g_mutex_init (&self->priv->mutex);
   self->priv->do_timestamp = TRUE;
 }
 
@@ -274,12 +281,12 @@ fs_multicast_transmitter_constructed (GObject *object)
 
     /* Lets create the RTP source funnel */
 
-    self->priv->udpsrc_funnels[c] = gst_element_factory_make ("fsfunnel", NULL);
+    self->priv->udpsrc_funnels[c] = gst_element_factory_make ("funnel", NULL);
 
     if (!self->priv->udpsrc_funnels[c]) {
       trans->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION,
-        "Could not make the fsfunnel element");
+        "Could not make the funnel element");
       return;
     }
 
@@ -287,11 +294,11 @@ fs_multicast_transmitter_constructed (GObject *object)
         self->priv->udpsrc_funnels[c])) {
       trans->construction_error = g_error_new (FS_ERROR,
         FS_ERROR_CONSTRUCTION,
-        "Could not add the fsfunnel element to the transmitter src bin");
+        "Could not add the funnel element to the transmitter src bin");
     }
 
     pad = gst_element_get_static_pad (self->priv->udpsrc_funnels[c], "src");
-    padname = g_strdup_printf ("src%d", c);
+    padname = g_strdup_printf ("src_%u", c);
     ghostpad = gst_ghost_pad_new (padname, pad);
     g_free (padname);
     gst_object_unref (pad);
@@ -319,7 +326,7 @@ fs_multicast_transmitter_constructed (GObject *object)
     }
 
     pad = gst_element_get_static_pad (self->priv->udpsink_tees[c], "sink");
-    padname = g_strdup_printf ("sink%d", c);
+    padname = g_strdup_printf ("sink_%u", c);
     ghostpad = gst_ghost_pad_new (padname, pad);
     g_free (padname);
     gst_object_unref (pad);
@@ -350,7 +357,7 @@ fs_multicast_transmitter_constructed (GObject *object)
         "sync" , FALSE,
         NULL);
 
-    pad = gst_element_get_request_pad (self->priv->udpsink_tees[c], "src%d");
+    pad = gst_element_get_request_pad (self->priv->udpsink_tees[c], "src_%u");
     pad2 = gst_element_get_static_pad (fakesink, "sink");
 
     ret = gst_pad_link (pad, pad2);
@@ -415,7 +422,7 @@ fs_multicast_transmitter_finalize (GObject *object)
     self->priv->udpsocks = NULL;
   }
 
-  g_mutex_free (self->priv->mutex);
+  g_mutex_clear (&self->priv->mutex);
 
   parent_class->finalize (object);
 }
@@ -439,9 +446,9 @@ fs_multicast_transmitter_get_property (GObject *object,
       g_value_set_uint (value, self->components);
       break;
     case PROP_TYPE_OF_SERVICE:
-      g_mutex_lock (self->priv->mutex);
+      FS_MULTICAST_TRANSMITTER_LOCK (self);
       g_value_set_uint (value, self->priv->type_of_service);
-      g_mutex_unlock (self->priv->mutex);
+      FS_MULTICAST_TRANSMITTER_UNLOCK (self);
       break;
     case PROP_DO_TIMESTAMP:
       g_value_set_boolean (value, self->priv->do_timestamp);
@@ -525,6 +532,7 @@ struct _UdpSock {
   guint8 current_ttl;
 
   gint fd;
+  GSocket *socket;
 
   /* Protected by the transmitter mutex */
   GByteArray *ttls;
@@ -698,7 +706,7 @@ _bind_port (
 
 static GstElement *
 _create_sinksource (gchar *elementname, GstBin *bin,
-    GstElement *teefunnel, GstElement *filter, gint fd,
+    GstElement *teefunnel, GstElement *filter, GSocket *socket,
     GstPadDirection direction, GstPad **requested_pad, GError **error)
 {
   GstElement *elem;
@@ -716,8 +724,8 @@ _create_sinksource (gchar *elementname, GstBin *bin,
   }
 
   g_object_set (elem,
-    "closefd", FALSE,
-    "sockfd", fd,
+    "close-socket", FALSE,
+    "socket", socket,
     "auto-multicast", FALSE,
     NULL);
 
@@ -730,9 +738,9 @@ _create_sinksource (gchar *elementname, GstBin *bin,
   }
 
   if (direction == GST_PAD_SINK)
-    *requested_pad = gst_element_get_request_pad (teefunnel, "src%d");
+    *requested_pad = gst_element_get_request_pad (teefunnel, "src_%u");
   else
-    *requested_pad = gst_element_get_request_pad (teefunnel, "sink%d");
+    *requested_pad = gst_element_get_request_pad (teefunnel, "sink_%u");
 
   if (!*requested_pad) {
     g_set_error (error, FS_ERROR, FS_ERROR_CONSTRUCTION,
@@ -910,11 +918,11 @@ fs_multicast_transmitter_get_udpsock (FsMulticastTransmitter *trans,
     return NULL;
   }
 
-  g_mutex_lock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_LOCK (trans);
   udpsock = fs_multicast_transmitter_get_udpsock_locked (trans, component_id,
       local_ip, multicast_ip, port, ttl, sending, &local_error);
   tos = trans->priv->type_of_service;
-  g_mutex_unlock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_UNLOCK (trans);
 
   if (local_error)
   {
@@ -946,13 +954,17 @@ fs_multicast_transmitter_get_udpsock (FsMulticastTransmitter *trans,
   if (udpsock->fd < 0)
     goto error;
 
+  udpsock->socket = g_socket_new_from_fd (udpsock->fd, error);
+  if (!udpsock->socket)
+    goto error;
+
   /* Now lets create the elements */
 
   udpsock->tee = trans->priv->udpsink_tees[component_id];
   udpsock->funnel = trans->priv->udpsrc_funnels[component_id];
 
   udpsock->udpsrc = _create_sinksource ("udpsrc",
-      GST_BIN (trans->priv->gst_src), udpsock->funnel, NULL, udpsock->fd,
+      GST_BIN (trans->priv->gst_src), udpsock->funnel, NULL, udpsock->socket,
       GST_PAD_SRC, &udpsock->udpsrc_requested_pad, error);
   if (!udpsock->udpsrc)
     goto error;
@@ -963,7 +975,7 @@ fs_multicast_transmitter_get_udpsock (FsMulticastTransmitter *trans,
   udpsock->udpsink = _create_sinksource ("multiudpsink",
       GST_BIN (trans->priv->gst_sink), udpsock->tee,
       udpsock->udpsink_recvonly_filter,
-      udpsock->fd, GST_PAD_SINK, &udpsock->udpsink_requested_pad, error);
+      udpsock->socket, GST_PAD_SINK, &udpsock->udpsink_requested_pad, error);
   if (!udpsock->udpsink)
     goto error;
 
@@ -972,14 +984,14 @@ fs_multicast_transmitter_get_udpsock (FsMulticastTransmitter *trans,
       "sync", FALSE,
       NULL);
 
-  g_mutex_lock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_LOCK (trans);
   /* Check if someone else has added the same thing at the same time */
   tmpudpsock = fs_multicast_transmitter_get_udpsock_locked (trans, component_id,
       local_ip, multicast_ip, port, ttl, sending, &local_error);
 
   if (tmpudpsock || local_error)
   {
-    g_mutex_unlock (trans->priv->mutex);
+    FS_MULTICAST_TRANSMITTER_UNLOCK (trans);
     fs_multicast_transmitter_put_udpsock (trans, udpsock, ttl);
     if (local_error)
     {
@@ -993,7 +1005,7 @@ fs_multicast_transmitter_get_udpsock (FsMulticastTransmitter *trans,
 
   trans->priv->udpsocks[component_id] =
     g_list_prepend (trans->priv->udpsocks[component_id], udpsock);
-  g_mutex_unlock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_UNLOCK (trans);
 
   if (udpsock->udpsink_recvonly_filter)
   {
@@ -1021,7 +1033,7 @@ fs_multicast_transmitter_put_udpsock (FsMulticastTransmitter *trans,
 {
   guint i;
 
-  g_mutex_lock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_LOCK (trans);
   for (i = udpsock->ttls->len - 1;; i--)
   {
     if (udpsock->ttls->data[i] == ttl)
@@ -1053,20 +1065,20 @@ fs_multicast_transmitter_put_udpsock (FsMulticastTransmitter *trans,
         {
           GST_WARNING ("Error setting the multicast TTL to %u: %s", max,
               g_strerror (errno));
-          g_mutex_unlock (trans->priv->mutex);
+          FS_MULTICAST_TRANSMITTER_UNLOCK (trans);
           return;
         }
         udpsock->current_ttl = max;
       }
     }
-    g_mutex_unlock (trans->priv->mutex);
+    FS_MULTICAST_TRANSMITTER_UNLOCK (trans);
     return;
   }
 
   trans->priv->udpsocks[udpsock->component_id] =
     g_list_remove (trans->priv->udpsocks[udpsock->component_id], udpsock);
 
-  g_mutex_unlock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_UNLOCK (trans);
 
   if (udpsock->udpsrc)
   {
@@ -1120,6 +1132,9 @@ fs_multicast_transmitter_put_udpsock (FsMulticastTransmitter *trans,
       GST_ERROR ("Could not remove sink filter element from transmitter sink");
   }
 
+  if (udpsock->socket)
+    g_object_unref (udpsock->socket);
+
   if (udpsock->fd >= 0)
     close (udpsock->fd);
 
@@ -1172,9 +1187,9 @@ void
 fs_multicast_transmitter_udpsock_ref (FsMulticastTransmitter *trans,
     UdpSock *udpsock, guint8 ttl)
 {
-  g_mutex_lock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_LOCK (trans);
   g_byte_array_append (udpsock->ttls, &ttl, 1);
-  g_mutex_unlock (trans->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_UNLOCK (trans);
 }
 
 
@@ -1184,7 +1199,7 @@ fs_multicast_transmitter_set_type_of_service (FsMulticastTransmitter *self,
 {
   gint i;
 
-  g_mutex_lock (self->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_LOCK (self);
   if (self->priv->type_of_service == tos)
     goto out;
 
@@ -1211,5 +1226,5 @@ fs_multicast_transmitter_set_type_of_service (FsMulticastTransmitter *self,
   }
 
  out:
-  g_mutex_unlock (self->priv->mutex);
+  FS_MULTICAST_TRANSMITTER_UNLOCK (self);
 }
